@@ -1,4 +1,4 @@
-/*	$OpenBSD: mrtparser.c,v 1.14 2021/01/18 12:16:09 claudio Exp $ */
+/*	$OpenBSD: mrtparser.c,v 1.16 2021/09/22 18:17:49 claudio Exp $ */
 /*
  * Copyright (c) 2011 Claudio Jeker <claudio@openbsd.org>
  *
@@ -163,6 +163,11 @@ mrt_parse(int fd, struct mrt_parser *p, int verbose)
 			case MRT_DUMP_V2_RIB_IPV6_UNICAST:
 			case MRT_DUMP_V2_RIB_IPV6_MULTICAST:
 			case MRT_DUMP_V2_RIB_GENERIC:
+			case MRT_DUMP_V2_RIB_IPV4_UNICAST_ADDPATH:
+			case MRT_DUMP_V2_RIB_IPV4_MULTICAST_ADDPATH:
+			case MRT_DUMP_V2_RIB_IPV6_UNICAST_ADDPATH:
+			case MRT_DUMP_V2_RIB_IPV6_MULTICAST_ADDPATH:
+			case MRT_DUMP_V2_RIB_GENERIC_ADDPATH:
 				if (p->dump == NULL)
 					break;
 				r = mrt_parse_v2_rib(&h, msg, verbose);
@@ -194,6 +199,10 @@ mrt_parse(int fd, struct mrt_parser *p, int verbose)
 			case BGP4MP_MESSAGE_AS4:
 			case BGP4MP_MESSAGE_LOCAL:
 			case BGP4MP_MESSAGE_AS4_LOCAL:
+			case BGP4MP_MESSAGE_ADDPATH:
+			case BGP4MP_MESSAGE_AS4_ADDPATH:
+			case BGP4MP_MESSAGE_LOCAL_ADDPATH:
+			case BGP4MP_MESSAGE_AS4_LOCAL_ADDPATH:
 				if ((m = mrt_parse_msg(&h, msg, verbose))) {
 					if (p->message)
 						p->message(m, p->arg);
@@ -362,7 +371,7 @@ mrt_parse_v2_rib(struct mrt_hdr *hdr, void *msg, int verbose)
 	struct mrt_rib	*r;
 	u_int8_t	*b = msg;
 	u_int		len = ntohl(hdr->length);
-	u_int32_t	snum;
+	u_int32_t	snum, path_id = 0;
 	u_int16_t	cnt, i, afi;
 	u_int8_t	safi, aid;
 	int		ret;
@@ -381,6 +390,10 @@ mrt_parse_v2_rib(struct mrt_hdr *hdr, void *msg, int verbose)
 	r->seqnum = ntohl(snum);
 
 	switch (ntohs(hdr->subtype)) {
+	case MRT_DUMP_V2_RIB_IPV4_UNICAST_ADDPATH:
+	case MRT_DUMP_V2_RIB_IPV4_MULTICAST_ADDPATH:
+		r->add_path = 1;
+		/* FALLTHROUGH */
 	case MRT_DUMP_V2_RIB_IPV4_UNICAST:
 	case MRT_DUMP_V2_RIB_IPV4_MULTICAST:
 		/* prefix */
@@ -389,6 +402,10 @@ mrt_parse_v2_rib(struct mrt_hdr *hdr, void *msg, int verbose)
 		if (ret == 1)
 			goto fail;
 		break;
+	case MRT_DUMP_V2_RIB_IPV6_UNICAST_ADDPATH:
+	case MRT_DUMP_V2_RIB_IPV6_MULTICAST_ADDPATH:
+		r->add_path = 1;
+		/* FALLTHROUGH */
 	case MRT_DUMP_V2_RIB_IPV6_UNICAST:
 	case MRT_DUMP_V2_RIB_IPV6_MULTICAST:
 		/* prefix */
@@ -397,8 +414,18 @@ mrt_parse_v2_rib(struct mrt_hdr *hdr, void *msg, int verbose)
 		if (ret == 1)
 			goto fail;
 		break;
+	case MRT_DUMP_V2_RIB_GENERIC_ADDPATH:
+		/*
+		 * RFC8050 handling for add-path has special handling for
+		 * RIB_GENERIC_ADDPATH but nobody implements it that way.
+		 * So just use the same way as for the other _ADDPATH types.
+		 */
+		r->add_path = 1;
+		/* FALLTHROUGH */
 	case MRT_DUMP_V2_RIB_GENERIC:
 		/* fetch AFI/SAFI pair */
+		if (len < 3)
+			goto fail;
 		memcpy(&afi, b, sizeof(afi));
 		b += sizeof(afi);
 		len -= sizeof(afi);
@@ -453,6 +480,16 @@ mrt_parse_v2_rib(struct mrt_hdr *hdr, void *msg, int verbose)
 		len -= sizeof(otm);
 		entries[i].originated = ntohl(otm);
 
+		if (r->add_path) {
+			if (len < sizeof(path_id) + sizeof(alen))
+				goto fail;
+			memcpy(&path_id, b, sizeof(path_id));
+			b += sizeof(path_id);
+			len -= sizeof(path_id);
+			path_id = ntohl(path_id);
+		}
+		entries[i].path_id = path_id;
+
 		/* attr_len */
 		memcpy(&alen, b, sizeof(alen));
 		b += sizeof(alen);
@@ -1141,7 +1178,7 @@ mrt_parse_msg(struct mrt_hdr *hdr, void *msg, int verbose)
 	u_int			 len = ntohl(hdr->length);
 	u_int32_t		 sas, das, usec;
 	u_int16_t		 tmp16, afi;
-	int			 r;
+	int			 r, addpath = 0;
 	u_int8_t		 aid;
 
 	t.tv_sec = ntohl(hdr->timestamp);
@@ -1156,7 +1193,12 @@ mrt_parse_msg(struct mrt_hdr *hdr, void *msg, int verbose)
 	}
 
 	switch (ntohs(hdr->subtype)) {
+	case BGP4MP_MESSAGE_ADDPATH:
+	case BGP4MP_MESSAGE_LOCAL_ADDPATH:
+		addpath = 1;
+		/* FALLTHROUGH */
 	case BGP4MP_MESSAGE:
+	case BGP4MP_MESSAGE_LOCAL:
 		if (len < 8)
 			return (0);
 		/* source as */
@@ -1178,7 +1220,12 @@ mrt_parse_msg(struct mrt_hdr *hdr, void *msg, int verbose)
 		len -= sizeof(tmp16);
 		afi = ntohs(tmp16);
 		break;
+	case BGP4MP_MESSAGE_AS4_ADDPATH:
+	case BGP4MP_MESSAGE_AS4_LOCAL_ADDPATH:
+		addpath = 1;
+		/* FALLTHROUGH */
 	case BGP4MP_MESSAGE_AS4:
+	case BGP4MP_MESSAGE_AS4_LOCAL:
 		if (len < 12)
 			return (0);
 		/* source as */
@@ -1213,6 +1260,7 @@ mrt_parse_msg(struct mrt_hdr *hdr, void *msg, int verbose)
 	m->time = t;
 	m->src_as = sas;
 	m->dst_as = das;
+	m->add_path = addpath;
 
 	if ((r = mrt_extract_addr(b, len, &m->src, aid)) == -1)
 		goto fail;
