@@ -1,4 +1,4 @@
-/*	$OpenBSD: bgpd.h,v 1.415 2021/06/17 16:05:26 claudio Exp $ */
+/*	$OpenBSD: bgpd.h,v 1.418 2021/09/01 12:39:52 claudio Exp $ */
 
 /*
  * Copyright (c) 2003, 2004 Henning Brauer <henning@openbsd.org>
@@ -105,6 +105,7 @@
 #define	F_CTL_OVS_INVALID	0x100000
 #define	F_CTL_OVS_NOTFOUND	0x200000
 #define	F_CTL_NEIGHBORS		0x400000 /* only used by bgpctl */
+#define	F_CTL_HAS_PATHID	0x800000 /* only set on requests */
 
 #define CTASSERT(x)	extern char  _ctassert[(x) ? 1 : -1 ] \
 			    __attribute__((__unused__))
@@ -266,6 +267,7 @@ struct roa {
 	uint8_t		maxlen;
 	uint8_t		pad;
 	uint32_t	asnum;
+	time_t		expires;
 	union {
 		struct in_addr	inet;
 		struct in6_addr	inet6;
@@ -801,6 +803,7 @@ struct ctl_neighbor {
 #define	F_PREF_ANNOUNCE	0x08
 #define	F_PREF_STALE	0x10
 #define	F_PREF_INVALID	0x20
+#define	F_PREF_PATH_ID	0x40
 
 struct ctl_show_rib {
 	struct bgpd_addr	true_nexthop;
@@ -810,6 +813,7 @@ struct ctl_show_rib {
 	char			descr[PEER_DESCR_LEN];
 	time_t			age;
 	u_int32_t		remote_id;
+	u_int32_t		path_id;
 	u_int32_t		local_pref;
 	u_int32_t		med;
 	u_int32_t		weight;
@@ -899,9 +903,10 @@ struct ctl_show_rib_request {
 	struct filter_as	as;
 	struct community	community;
 	u_int32_t		flags;
-	u_int8_t		validation_state;
+	u_int32_t		path_id;
 	pid_t			pid;
 	enum imsg_type		type;
+	u_int8_t		validation_state;
 	u_int8_t		prefixlen;
 	u_int8_t		aid;
 };
