@@ -1,4 +1,4 @@
-/*	$OpenBSD: rde.h,v 1.240 2021/06/17 16:05:26 claudio Exp $ */
+/*	$OpenBSD: rde.h,v 1.242 2021/08/09 08:15:34 claudio Exp $ */
 
 /*
  * Copyright (c) 2003, 2004 Claudio Jeker <claudio@openbsd.org> and
@@ -56,10 +56,10 @@ struct rib {
 	struct filter_head	*in_rules_tmp;
 	u_int			rtableid;
 	u_int			rtableid_tmp;
+	enum reconf_action	state, fibstate;
+	u_int16_t		id;
 	u_int16_t		flags;
 	u_int16_t		flags_tmp;
-	u_int16_t		id;
-	enum reconf_action	state, fibstate;
 };
 
 #define RIB_ADJ_IN	0
@@ -317,18 +317,20 @@ struct prefix {
 	union {
 		struct {
 			LIST_ENTRY(prefix)	 rib, nexthop;
+			struct rib_entry	*re;
 		} list;
 		struct {
 			RB_ENTRY(prefix)	 index, update;
 		} tree;
 	}				 entry;
 	struct pt_entry			*pt;
-	struct rib_entry		*re;
 	struct rde_aspath		*aspath;
 	struct rde_community		*communities;
 	struct rde_peer			*peer;
 	struct nexthop			*nexthop;	/* may be NULL */
 	time_t				 lastchange;
+	u_int32_t			 path_id;
+	u_int32_t			 path_id_tx;
 	u_int8_t			 validation_state;
 	u_int8_t			 nhflags;
 	u_int8_t			 eor;
@@ -338,6 +340,7 @@ struct prefix {
 #define	PREFIX_FLAG_DEAD	0x04	/* locked but removed */
 #define	PREFIX_FLAG_STALE	0x08	/* stale entry (graceful reload) */
 #define	PREFIX_FLAG_MASK	0x0f	/* mask for the prefix types */
+#define	PREFIX_FLAG_ADJOUT	0x10	/* prefix is in the adj-out rib */
 #define	PREFIX_NEXTHOP_LINKED	0x40	/* prefix is linked onto nexthop list */
 #define	PREFIX_FLAG_LOCKED	0x80	/* locked by rib walker */
 };
@@ -385,6 +388,7 @@ int		rde_match_peer(struct rde_peer *, struct ctl_neighbor *);
 
 /* rde_peer.c */
 int		 peer_has_as4byte(struct rde_peer *);
+int		 peer_has_add_path(struct rde_peer *, u_int8_t, int);
 int		 peer_accept_no_as_set(struct rde_peer *);
 void		 peer_init(u_int32_t);
 void		 peer_shutdown(void);
@@ -576,13 +580,13 @@ void		 path_clean(struct rde_aspath *);
 void		 path_put(struct rde_aspath *);
 
 #define	PREFIX_SIZE(x)	(((x) + 7) / 8 + 1)
-struct prefix	*prefix_get(struct rib *, struct rde_peer *,
+struct prefix	*prefix_get(struct rib *, struct rde_peer *, u_int32_t,
 		    struct bgpd_addr *, int);
 struct prefix	*prefix_lookup(struct rde_peer *, struct bgpd_addr *, int);
 struct prefix	*prefix_match(struct rde_peer *, struct bgpd_addr *);
-int		 prefix_update(struct rib *, struct rde_peer *,
+int		 prefix_update(struct rib *, struct rde_peer *, u_int32_t,
 		     struct filterstate *, struct bgpd_addr *, int, u_int8_t);
-int		 prefix_withdraw(struct rib *, struct rde_peer *,
+int		 prefix_withdraw(struct rib *, struct rde_peer *, u_int32_t,
 		    struct bgpd_addr *, int);
 void		 prefix_add_eor(struct rde_peer *, u_int8_t);
 int		 prefix_adjout_update(struct rde_peer *, struct filterstate *,
@@ -597,7 +601,8 @@ int		 prefix_dump_new(struct rde_peer *, u_int8_t, unsigned int,
 		    void (*)(void *, u_int8_t), int (*)(void *));
 int		 prefix_write(u_char *, int, struct bgpd_addr *, u_int8_t, int);
 int		 prefix_writebuf(struct ibuf *, struct bgpd_addr *, u_int8_t);
-struct prefix	*prefix_bypeer(struct rib_entry *, struct rde_peer *);
+struct prefix	*prefix_bypeer(struct rib_entry *, struct rde_peer *,
+		    u_int32_t);
 void		 prefix_destroy(struct prefix *);
 void		 prefix_relink(struct prefix *, struct rde_aspath *, int);
 
@@ -639,6 +644,14 @@ prefix_vstate(struct prefix *p)
 	return (p->validation_state & ROA_MASK);
 }
 
+static inline struct rib_entry *
+prefix_re(struct prefix *p)
+{
+	if (p->flags & PREFIX_FLAG_ADJOUT)
+		return NULL;
+	return (p->entry.list.re);
+}
+
 void		 nexthop_init(u_int32_t);
 void		 nexthop_shutdown(void);
 int		 nexthop_pending(void);
