@@ -1,4 +1,4 @@
-/*	$OpenBSD: rde_rib.c,v 1.222 2021/06/17 08:16:04 claudio Exp $ */
+/*	$OpenBSD: rde_rib.c,v 1.224 2021/08/09 08:15:35 claudio Exp $ */
 
 /*
  * Copyright (c) 2003, 2004 Claudio Jeker <claudio@openbsd.org>
@@ -842,7 +842,7 @@ path_put(struct rde_aspath *asp)
 /* prefix specific functions */
 
 static int	prefix_add(struct bgpd_addr *, int, struct rib *,
-		    struct rde_peer *, struct rde_aspath *,
+		    struct rde_peer *, u_int32_t, struct rde_aspath *,
 		    struct rde_community *, struct nexthop *,
 		    u_int8_t, u_int8_t);
 static int	prefix_move(struct prefix *, struct rde_peer *,
@@ -850,7 +850,7 @@ static int	prefix_move(struct prefix *, struct rde_peer *,
 		    struct nexthop *, u_int8_t, u_int8_t);
 
 static void	prefix_link(struct prefix *, struct rib_entry *,
-		     struct rde_peer *, struct rde_aspath *,
+		     struct rde_peer *, u_int32_t, struct rde_aspath *,
 		     struct rde_community *, struct nexthop *,
 		     u_int8_t, u_int8_t);
 static void	prefix_unlink(struct prefix *);
@@ -876,12 +876,14 @@ prefix_cmp(struct prefix *a, struct prefix *b)
 		return (a->nexthop > b->nexthop ? 1 : -1);
 	if (a->nhflags != b->nhflags)
 		return (a->nhflags > b->nhflags ? 1 : -1);
+	/* XXX path_id ??? */
 	return pt_prefix_cmp(a->pt, b->pt);
 }
 
 static inline int
 prefix_index_cmp(struct prefix *a, struct prefix *b)
 {
+	/* XXX path_id ??? */
 	return pt_prefix_cmp(a->pt, b->pt);
 }
 
@@ -892,15 +894,15 @@ RB_GENERATE_STATIC(prefix_index, prefix, entry.tree.index, prefix_index_cmp)
  * search for specified prefix of a peer. Returns NULL if not found.
  */
 struct prefix *
-prefix_get(struct rib *rib, struct rde_peer *peer, struct bgpd_addr *prefix,
-    int prefixlen)
+prefix_get(struct rib *rib, struct rde_peer *peer, u_int32_t path_id,
+    struct bgpd_addr *prefix, int prefixlen)
 {
 	struct rib_entry	*re;
 
 	re = rib_get(rib, prefix, prefixlen);
 	if (re == NULL)
 		return (NULL);
-	return (prefix_bypeer(re, peer));
+	return (prefix_bypeer(re, peer, path_id));
 }
 
 /*
@@ -954,8 +956,9 @@ prefix_match(struct rde_peer *peer, struct bgpd_addr *addr)
  * Return 1 if prefix was newly added, 0 if it was just changed.
  */
 int
-prefix_update(struct rib *rib, struct rde_peer *peer, struct filterstate *state,
-    struct bgpd_addr *prefix, int prefixlen, u_int8_t vstate)
+prefix_update(struct rib *rib, struct rde_peer *peer, u_int32_t path_id,
+    struct filterstate *state, struct bgpd_addr *prefix, int prefixlen,
+    u_int8_t vstate)
 {
 	struct rde_aspath	*asp, *nasp = &state->aspath;
 	struct rde_community	*comm, *ncomm = &state->communities;
@@ -964,7 +967,7 @@ prefix_update(struct rib *rib, struct rde_peer *peer, struct filterstate *state,
 	/*
 	 * First try to find a prefix in the specified RIB.
 	 */
-	if ((p = prefix_get(rib, peer, prefix, prefixlen)) != NULL) {
+	if ((p = prefix_get(rib, peer, path_id, prefix, prefixlen)) != NULL) {
 		if (prefix_nexthop(p) == state->nexthop &&
 		    prefix_nhflags(p) == state->nhflags &&
 		    communities_equal(ncomm, prefix_communities(p)) &&
@@ -997,8 +1000,8 @@ prefix_update(struct rib *rib, struct rde_peer *peer, struct filterstate *state,
 		return (prefix_move(p, peer, asp, comm, state->nexthop,
 		    state->nhflags, vstate));
 	else
-		return (prefix_add(prefix, prefixlen, rib, peer, asp, comm,
-		    state->nexthop, state->nhflags, vstate));
+		return (prefix_add(prefix, prefixlen, rib, peer, path_id, asp,
+		    comm, state->nexthop, state->nhflags, vstate));
 }
 
 /*
@@ -1006,8 +1009,9 @@ prefix_update(struct rib *rib, struct rde_peer *peer, struct filterstate *state,
  */
 static int
 prefix_add(struct bgpd_addr *prefix, int prefixlen, struct rib *rib,
-    struct rde_peer *peer, struct rde_aspath *asp, struct rde_community *comm,
-    struct nexthop *nexthop, u_int8_t nhflags, u_int8_t vstate)
+    struct rde_peer *peer, u_int32_t path_id, struct rde_aspath *asp,
+    struct rde_community *comm, struct nexthop *nexthop, u_int8_t nhflags,
+    u_int8_t vstate)
 {
 	struct prefix		*p;
 	struct rib_entry	*re;
@@ -1017,7 +1021,7 @@ prefix_add(struct bgpd_addr *prefix, int prefixlen, struct rib *rib,
 		re = rib_add(rib, prefix, prefixlen);
 
 	p = prefix_alloc();
-	prefix_link(p, re, peer, asp, comm, nexthop, nhflags, vstate);
+	prefix_link(p, re, peer, path_id, asp, comm, nexthop, nhflags, vstate);
 	return (1);
 }
 
@@ -1031,6 +1035,9 @@ prefix_move(struct prefix *p, struct rde_peer *peer,
 {
 	struct prefix		*np;
 
+	if (p->flags & PREFIX_FLAG_ADJOUT)
+		fatalx("%s: prefix with PREFIX_FLAG_ADJOUT hit", __func__);
+
 	if (peer != prefix_peer(p))
 		fatalx("prefix_move: cross peer move");
 
@@ -1040,8 +1047,9 @@ prefix_move(struct prefix *p, struct rde_peer *peer,
 	np->aspath = path_ref(asp);
 	np->communities = communities_ref(comm);
 	np->peer = peer;
-	np->re = p->re;
+	np->entry.list.re = prefix_re(p);
 	np->pt = p->pt; /* skip refcnt update since ref is moved */
+	np->path_id = p->path_id;
 	np->validation_state = vstate;
 	np->nhflags = nhflags;
 	np->nexthop = nexthop_ref(nexthop);
@@ -1056,7 +1064,7 @@ prefix_move(struct prefix *p, struct rde_peer *peer,
 	 * no need to update the peer prefix count because we are only moving
 	 * the prefix without changing the peer.
 	 */
-	prefix_evaluate(np->re, np, p);
+	prefix_evaluate(prefix_re(np), np, p);
 
 	/* remove possible pftable reference from old path first */
 	if (p->aspath && p->aspath->pftableid)
@@ -1071,8 +1079,8 @@ prefix_move(struct prefix *p, struct rde_peer *peer,
 	p->nexthop = NULL;
 	p->aspath = NULL;
 	p->peer = NULL;
-	p->re = NULL;
 	p->pt = NULL;
+	p->entry.list.re = NULL;
 	prefix_free(p);
 
 	return (0);
@@ -1083,16 +1091,18 @@ prefix_move(struct prefix *p, struct rde_peer *peer,
  * or pt_entry -- become empty remove them too.
  */
 int
-prefix_withdraw(struct rib *rib, struct rde_peer *peer,
+prefix_withdraw(struct rib *rib, struct rde_peer *peer, u_int32_t path_id,
     struct bgpd_addr *prefix, int prefixlen)
 {
 	struct prefix		*p;
 	struct rde_aspath	*asp;
 
-	p = prefix_get(rib, peer, prefix, prefixlen);
+	p = prefix_get(rib, peer, path_id, prefix, prefixlen);
 	if (p == NULL)		/* Got a dummy withdrawn request. */
 		return (0);
 
+	if (p->flags & PREFIX_FLAG_ADJOUT)
+		fatalx("%s: prefix with PREFIX_FLAG_ADJOUT hit", __func__);
 	asp = prefix_aspath(p);
 	if (asp && asp->pftableid)
 		/* only prefixes in the local RIB were pushed into pf */
@@ -1112,8 +1122,8 @@ prefix_add_eor(struct rde_peer *peer, u_int8_t aid)
 	struct prefix *p;
 
 	p = prefix_alloc();
+	p->flags = PREFIX_FLAG_ADJOUT | PREFIX_FLAG_UPDATE;
 	p->eor = 1;
-	p->flags = PREFIX_FLAG_UPDATE;
 	if (RB_INSERT(prefix_tree, &peer->updates[aid], p) != NULL)
 		/* no need to add if EoR marker already present */
 		prefix_free(p);
@@ -1134,6 +1144,9 @@ prefix_adjout_update(struct rde_peer *peer, struct filterstate *state,
 	int created = 0;
 
 	if ((p = prefix_lookup(peer, prefix, prefixlen)) != NULL) {
+		if ((p->flags & PREFIX_FLAG_ADJOUT) == 0)
+			fatalx("%s: prefix without PREFIX_FLAG_ADJOUT hit",
+			    __func__);
 		/* prefix is already in the Adj-RIB-Out */
 		if (p->flags & PREFIX_FLAG_WITHDRAW) {
 			created = 1;	/* consider this a new entry */
@@ -1171,6 +1184,7 @@ prefix_adjout_update(struct rde_peer *peer, struct filterstate *state,
 		/* peer and pt remain */
 	} else {
 		p = prefix_alloc();
+		p->flags |= PREFIX_FLAG_ADJOUT;
 		created = 1;
 
 		p->pt = pt_get(prefix, prefixlen);
@@ -1225,6 +1239,9 @@ prefix_adjout_withdraw(struct rde_peer *peer, struct bgpd_addr *prefix,
 	if (p == NULL)		/* Got a dummy withdrawn request. */
 		return (0);
 
+	if ((p->flags & PREFIX_FLAG_ADJOUT) == 0)
+		fatalx("%s: prefix without PREFIX_FLAG_ADJOUT hit", __func__);
+
 	/* already a withdraw, shortcut */
 	if (p->flags & PREFIX_FLAG_WITHDRAW) {
 		p->lastchange = getmonotime();
@@ -1285,6 +1302,9 @@ prefix_adjout_destroy(struct prefix *p)
 {
 	struct rde_peer *peer = prefix_peer(p);
 
+	if ((p->flags & PREFIX_FLAG_ADJOUT) == 0)
+		fatalx("%s: prefix without PREFIX_FLAG_ADJOUT hit", __func__);
+
 	if (p->eor) {
 		/* EOR marker is not linked in the index */
 		prefix_free(p);
@@ -1472,12 +1492,12 @@ prefix_writebuf(struct ibuf *buf, struct bgpd_addr *prefix, u_int8_t plen)
  * belonging to the peer peer. Returns NULL if no match found.
  */
 struct prefix *
-prefix_bypeer(struct rib_entry *re, struct rde_peer *peer)
+prefix_bypeer(struct rib_entry *re, struct rde_peer *peer, u_int32_t path_id)
 {
 	struct prefix	*p;
 
 	LIST_FOREACH(p, &re->prefix_h, entry.list.rib)
-		if (prefix_peer(p) == peer)
+		if (prefix_peer(p) == peer && p->path_id == path_id)
 			return (p);
 	return (NULL);
 }
@@ -1486,8 +1506,10 @@ static void
 prefix_evaluate_all(struct prefix *p, enum nexthop_state state,
     enum nexthop_state oldstate)
 {
+	struct rib_entry *re = prefix_re(p);
+
 	/* Skip non local-RIBs or RIBs that are flagged as noeval. */
-	if (re_rib(p->re)->flags & F_RIB_NOEVALUATE) {
+	if (re_rib(re)->flags & F_RIB_NOEVALUATE) {
 		log_warnx("%s: prefix with F_RIB_NOEVALUATE hit", __func__);
 		return;
 	}
@@ -1500,15 +1522,15 @@ prefix_evaluate_all(struct prefix *p, enum nexthop_state state,
 		 * the routing decision so shortcut here.
 		 */
 		if (state == NEXTHOP_REACH) {
-			if ((re_rib(p->re)->flags & F_RIB_NOFIB) == 0 &&
-			    p == p->re->active)
-				rde_send_kroute(re_rib(p->re), p, NULL);
+			if ((re_rib(re)->flags & F_RIB_NOFIB) == 0 &&
+			    p == re->active)
+				rde_send_kroute(re_rib(re), p, NULL);
 		}
 		return;
 	}
 
 	/* redo the route decision */
-	prefix_evaluate(p->re, p, p);
+	prefix_evaluate(prefix_re(p), p, p);
 }
 
 /* kill a prefix. */
@@ -1516,7 +1538,7 @@ void
 prefix_destroy(struct prefix *p)
 {
 	/* make route decision */
-	prefix_evaluate(p->re, NULL, p);
+	prefix_evaluate(prefix_re(p), NULL, p);
 
 	prefix_unlink(p);
 	prefix_free(p);
@@ -1527,14 +1549,18 @@ prefix_destroy(struct prefix *p)
  */
 static void
 prefix_link(struct prefix *p, struct rib_entry *re, struct rde_peer *peer,
-    struct rde_aspath *asp, struct rde_community *comm,
+    u_int32_t path_id, struct rde_aspath *asp, struct rde_community *comm,
     struct nexthop *nexthop, u_int8_t nhflags, u_int8_t vstate)
 {
+	if (p->flags & PREFIX_FLAG_ADJOUT)
+		fatalx("%s: prefix with PREFIX_FLAG_ADJOUT hit", __func__);
+
+	p->entry.list.re = re;
 	p->aspath = path_ref(asp);
 	p->communities = communities_ref(comm);
 	p->peer = peer;
 	p->pt = pt_ref(re->prefix);
-	p->re = re;
+	p->path_id = path_id;
 	p->validation_state = vstate;
 	p->nhflags = nhflags;
 	p->nexthop = nexthop_ref(nexthop);
@@ -1555,7 +1581,7 @@ prefix_link(struct prefix *p, struct rib_entry *re, struct rde_peer *peer,
 static void
 prefix_unlink(struct prefix *p)
 {
-	struct rib_entry	*re = p->re;
+	struct rib_entry	*re = prefix_re(p);
 
 	/* destroy all references to other objects */
 	nexthop_unlink(p);
@@ -1567,7 +1593,6 @@ prefix_unlink(struct prefix *p)
 	p->nexthop = NULL;
 	p->aspath = NULL;
 	p->peer = NULL;
-	p->re = NULL;
 	p->pt = NULL;
 
 	if (re && rib_empty(re))
@@ -1795,7 +1820,7 @@ nexthop_link(struct prefix *p)
 		return;
 
 	/* no need to link prefixes in RIBs that have no decision process */
-	if (re_rib(p->re)->flags & F_RIB_NOEVALUATE)
+	if (re_rib(prefix_re(p))->flags & F_RIB_NOEVALUATE)
 		return;
 
 	p->flags |= PREFIX_NEXTHOP_LINKED;
