@@ -1,4 +1,4 @@
-/*	$OpenBSD: rtr_proto.c,v 1.1 2021/02/16 08:29:16 claudio Exp $ */
+/*	$OpenBSD: rtr_proto.c,v 1.4 2021/08/02 16:42:13 claudio Exp $ */
 
 /*
  * Copyright (c) 2020 Claudio Jeker <claudio@openbsd.org>
@@ -449,6 +449,14 @@ rtr_parse_ipv4_prefix(struct rtr_session *rs, uint8_t *buf, size_t len)
 		rtr_send_error(rs, INTERNAL_ERROR, "out of memory", NULL, 0);
 		return -1;
 	}
+	if (ip4.prefixlen > 32 || ip4.maxlen > 32 ||
+	    ip4.prefixlen > ip4.maxlen) {
+		log_warnx("rtr: %s: received %s: bad prefixlen / maxlen",
+		    log_rtr(rs), log_rtr_type(IPV4_PREFIX));
+		rtr_send_error(rs, CORRUPT_DATA, "bad prefixlen / maxlen",
+		    buf, len);
+		return -1;
+	}
 	roa->aid = AID_INET;
 	roa->prefixlen = ip4.prefixlen;
 	roa->maxlen = ip4.maxlen;
@@ -510,6 +518,14 @@ rtr_parse_ipv6_prefix(struct rtr_session *rs, uint8_t *buf, size_t len)
 		rtr_send_error(rs, INTERNAL_ERROR, "out of memory", NULL, 0);
 		return -1;
 	}
+	if (ip6.prefixlen > 128 || ip6.maxlen > 128 ||
+	    ip6.prefixlen > ip6.maxlen) {
+		log_warnx("rtr: %s: received %s: bad prefixlen / maxlen",
+		    log_rtr(rs), log_rtr_type(IPV6_PREFIX));
+		rtr_send_error(rs, CORRUPT_DATA, "bad prefixlen / maxlen",
+		    buf, len);
+		return -1;
+	}
 	roa->aid = AID_INET6;
 	roa->prefixlen = ip6.prefixlen;
 	roa->maxlen = ip6.maxlen;
@@ -751,7 +767,7 @@ rtr_process_msg(struct rtr_session *rs)
 			break;
 		default:
 			log_warnx("rtr %s: received %s: unexpected pdu type",
-		    	    log_rtr(rs), log_rtr_type(msgtype));
+			    log_rtr(rs), log_rtr_type(msgtype));
 			rtr_send_error(rs, INVALID_REQUEST, NULL, rptr, msglen);
 			return;
 		}
@@ -1004,7 +1020,7 @@ struct rtr_session *
 rtr_new(uint32_t id, char *descr)
 {
 	struct rtr_session *rs;
-	
+
 	if ((rs = calloc(1, sizeof(*rs))) == NULL)
 		fatal("RTR session %s", descr);
 
